!
!###############################################################################
! ***** VERSION OF MICROPHYSICS DESIGNED FOR HIGHER RESOLUTION MESO ETA MODEL
!       (1) Represents sedimentation by preserving a portion of the precipitation
!           through top-down integration from cloud-top.  Modified procedure to
!           Zhao and Carr (1997).
!       (2) Microphysical equations are modified to be less sensitive to time
!           steps by use of Clausius-Clapeyron equation to account for changes in
!           saturation mixing ratios in response to latent heating/cooling.  
!       (3) Prevent spurious temperature oscillations across 0C due to 
!           microphysics.
!       (4) Uses lookup tables for: calculating two different ventilation 
!           coefficients in condensation and deposition processes; accretion of
!           cloud water by precipitation; precipitation mass; precipitation rate
!           (and mass-weighted precipitation fall speeds).
!       (5) Assumes temperature-dependent variation in mean diameter of large ice
!           (Houze et al., 1979; Ryan et al., 1996).
!        -> 8/22/01: This relationship has been extended to colder temperatures
!           to parameterize smaller large-ice particles down to mean sizes of MDImin,
!           which is 50 microns reached at -55.9C.
!       (6) Attempts to differentiate growth of large and small ice, mainly for
!           improved transition from thin cirrus to thick, precipitating ice
!           anvils.
!        -> 8/22/01: This feature has been diminished by effectively adjusting to
!           ice saturation during depositional growth at temperatures colder than
!           -10C.  Ice sublimation is calculated more explicitly.  The logic is
!           that sources of are either poorly understood (e.g., nucleation for NWP) 
!           or are not represented in the Eta model (e.g., detrainment of ice from 
!           convection).  Otherwise the model is too wet compared to the radiosonde
!           observations based on 1 Feb - 18 March 2001 retrospective runs.  
!       (7) Top-down integration also attempts to treat mixed-phase processes,
!           allowing a mixture of ice and water.  Based on numerous observational
!           studies, ice growth is based on nucleation at cloud top &
!           subsequent growth by vapor deposition and riming as the ice particles 
!           fall through the cloud.  Effective nucleation rates are a function
!           of ice supersaturation following Meyers et al. (JAM, 1992).  
!        -> 8/22/01: The simulated relative humidities were far too moist compared 
!           to the rawinsonde observations.  This feature has been substantially 
!           diminished, limited to a much narrower temperature range of 0 to -10C.  
!       (8) Depositional growth of newly nucleated ice is calculated for large time
!           steps using Fig. 8 of Miller and Young (JAS, 1979), at 1 deg intervals
!           using their ice crystal masses calculated after 600 s of growth in water
!           saturated conditions.  The growth rates are normalized by time step
!           assuming 3D growth with time**1.5 following eq. (6.3) in Young (1993).
!        -> 8/22/01: This feature has been effectively limited to 0 to -10C.  
!       (9) Ice precipitation rates can increase due to increase in response to
!           cloud water riming due to (a) increased density & mass of the rimed
!           ice, and (b) increased fall speeds of rimed ice.
!        -> 8/22/01: This feature has been effectively limited to 0 to -10C.  
!###############################################################################
!###############################################################################
!
      SUBROUTINE GSMCOLUMN (ARAIN, ASNOW, DTPH, lmh,
     &     P_col, QI_col, QR_col, QV_col, QW_col, RimeF_col, T_col, 
     &     THICK_col, WC_col)
!     
!###############################################################################
!###############################################################################
!
!-------------------------------------------------------------------------------
!----- NOTE:  Code is currently set up w/o threading!  
!-------------------------------------------------------------------------------
!$$$  SUBPROGRAM DOCUMENTATION BLOCK
!                .      .    .     
! SUBPROGRAM:  Grid-scale microphysical processes - condensation & precipitation
!   PRGRMMR: Ferrier         ORG: W/NP22     DATE: 08-2001
!-------------------------------------------------------------------------------
! ABSTRACT:
!   * Merges original GSCOND & PRECPD subroutines.   
!   * Code has been substantially streamlined and restructured.
!   * Exchange between water vapor & small cloud condensate is calculated using
!     the original Asai (1965, J. Japan) algorithm.  See also references to
!     Yau and Austin (1979, JAS), Rutledge and Hobbs (1983, JAS), and Tao et al.
!     (1989, MWR).  This algorithm replaces the Sundqvist et al. (1989, MWR)
!     parameterization.  
!-------------------------------------------------------------------------------
!     
! USAGE: 
!   * CALL GSMCOLUMN FROM SUBROUTINE GSMDRIVE
!   * SUBROUTINE GSMDRIVE CALLED FROM MAIN PROGRAM EBU
!
! INPUT ARGUMENT LIST:
!   DTPH       - physics time step (s)
!   I_index    - I index
!   J_index    - J index
!   LMH(I,J)       - Eta level of level above surface, ground
!   P_col      - vertical column of model pressure (Pa)
!   QI_col     - vertical column of model ice mixing ratio (kg/kg)
!   QR_col     - vertical column of model rain ratio (kg/kg)
!   QV_col     - vertical column of model water vapor specific humidity (kg/kg)
!   QW_col     - vertical column of model cloud water mixing ratio (kg/kg)
!   RimeF_col  - vertical column of rime factor for ice in model (ratio, defined below)
!   T_col      - vertical column of model temperature (deg K)
!   THICK_col  - vertical column of model mass thickness (density*height increment)
!   WC_col     - vertical column of model mixing ratio of total condensate (kg/kg)
!   
!
! OUTPUT ARGUMENT LIST: 
!   ARAIN      - accumulated rainfall at the surface (kg)
!   ASNOW      - accumulated snowfall at the surface (kg)
!   QV_col     - vertical column of model water vapor specific humidity (kg/kg)
!   WC_col     - vertical column of model mixing ratio of total condensate (kg/kg)
!   QW_col     - vertical column of model cloud water mixing ratio (kg/kg)
!   QI_col     - vertical column of model ice mixing ratio (kg/kg)
!   QR_col     - vertical column of model rain ratio (kg/kg)
!   RimeF_col  - vertical column of rime factor for ice in model (ratio, defined below)
!   T_col      - vertical column of model temperature (deg K)
!     
! OUTPUT FILES:
!     NONE
!     
! Subprograms & Functions called:
!   * Real Function CONDENSE  - cloud water condensation
!   * Real Function DEPOSIT   - ice deposition (not sublimation)
!
! UNIQUE: NONE
!  
! LIBRARY: NONE
!  
! COMMON BLOCKS:  
!     CMICRO_CONS  - key constants initialized in GSMCONST
!     CMICRO_STATS - accumulated and maximum statistics
!     CMY_GROWTH   - lookup table for growth of ice crystals in 
!                    water saturated conditions (Miller & Young, 1979)
!     IVENT_TABLES - lookup tables for ventilation effects of ice
!     IACCR_TABLES - lookup tables for accretion rates of ice
!     IMASS_TABLES - lookup tables for mass content of ice
!     IRATE_TABLES - lookup tables for precipitation rates of ice
!     IRIME_TABLES - lookup tables for increase in fall speed of rimed ice
!     RVENT_TABLES - lookup tables for ventilation effects of rain
!     RACCR_TABLES - lookup tables for accretion rates of rain
!     RMASS_TABLES - lookup tables for mass content of rain
!     RVELR_TABLES - lookup tables for fall speeds of rain
!     RRATE_TABLES - lookup tables for precipitation rates of rain
!   
! ATTRIBUTES:
!   LANGUAGE: FORTRAN 90
!   MACHINE : IBM SP
!
!
!------------------------------------------------------------------------- 
!--------------- Arrays & constants in argument list --------------------- 
!------------------------------------------------------------------------- 
!
      INCLUDE "parmeta"
      INCLUDE "mpp.h"
      REAL,    INTENT(OUT  ) :: arain(myis:myie, myjs:myje)
      REAL,    INTENT(OUT  ) :: asnow(myis:myie, myjs:myje)
      REAL,    INTENT(IN   ) :: dtph
      INTEGER, INTENT(IN   ) :: lmh(idim1:idim2, jdim1:jdim2)
      REAL,    INTENT(IN   ) :: p_col(myis:myie, myjs:myje, lm) 
      REAL,    INTENT(INOUT) :: qi_col(myis:myie, myjs:myje, lm)
      REAL,    INTENT(INOUT) :: qr_col(myis:myie, myjs:myje, lm)
      REAL,    INTENT(INOUT) :: qv_col(myis:myie, myjs:myje, lm)
      REAL,    INTENT(INOUT) :: qw_col(myis:myie, myjs:myje, lm)
      REAL,    INTENT(INOUT) :: rimef_col(myis:myie, myjs:myje, lm)
      REAL,    INTENT(INOUT) :: t_col(myis:myie, myjs:myje, lm)
      REAL,    INTENT(IN   ) :: thick_col(myis:myie, myjs:myje, lm)
      REAL,    INTENT(INOUT) :: wc_col(myis:myie, myjs:myje, lm)
!
!
!------------------------------------------------------------------------- 
!-------------- Common blocks for microphysical statistics ---------------
!------------------------------------------------------------------------- 
!
!------------------------------------------------------------------------- 
!--------- Common blocks for constants initialized in GSMCONST ----------
!
      COMMON /CMICRO_CONS/ ABFR, CBFR, CIACW, CIACR, C_N0r0, 
     & CN0r0, CN0r_DMRmin, CN0r_DMRmax, CRACW, CRAUT, ESW0, 
     & QAUT0, RFmax, RHgrd, RQR_DR1, RQR_DR2, RQR_DR3, RQR_DRmin, 
     & RQR_DRmax, RR_DRmin, RR_DR1, RR_DR2, RR_DR3, RR_DRmax
!
!--- The following variables are for microphysical statistics
!
      INTEGER, PARAMETER :: ITLO=-60, ITHI=40
      COMMON /CMICRO_STATS/ NSTATS(ITLO:ITHI,4), QMAX(ITLO:ITHI,5),
     & QTOT(ITLO:ITHI,22)
      INTEGER :: NSTATS
      REAL :: QMAX, QTOT
!
!------------------------------------------------------------------------- 
!--------------- Common blocks for various lookup tables -----------------
!
!--- Discretized growth rates of small ice crystals after their nucleation 
!     at 1 C intervals from -1 C to -35 C, based on calculations by Miller 
!     and Young (1979, JAS) after 600 s of growth.  Resultant growth rates
!     are multiplied by physics time step in GSMCONST.
!
      INTEGER, PARAMETER :: MY_T1=1, MY_T2=35
      COMMON /CMY600/ MY_GROWTH(MY_T1:MY_T2)
      REAL MY_GROWTH
!
!------------------------------------------------------------------------- 
!
!--- Mean ice-particle diameters varying from 50 microns to 1000 microns
!      (1 mm), assuming an exponential size distribution.  
!
!---- Meaning of the following arrays: 
!        - mdiam - mean diameter (m)
!        - VENTI1 - integrated quantity associated w/ ventilation effects 
!                   (capacitance only) for calculating vapor deposition onto ice
!        - VENTI2 - integrated quantity associated w/ ventilation effects 
!                   (with fall speed) for calculating vapor deposition onto ice
!        - ACCRI  - integrated quantity associated w/ cloud water collection by ice
!        - MASSI  - integrated quantity associated w/ ice mass 
!        - VSNOWI - mass-weighted fall speed of snow (large ice), used to calculate 
!                   precipitation rates
!
      REAL, PARAMETER :: DMImin=.05e-3, DMImax=1.e-3, DelDMI=1.e-6,
     &  XMImin=1.e6*DMImin, XMImax=1.e6*DMImax
      INTEGER, PARAMETER :: MDImin=XMImin, MDImax=XMImax
!
      COMMON /IACCR_TABLES/ ACCRI(MDImin:MDImax)
      COMMON /IMASS_TABLES/ MASSI(MDImin:MDImax)
      REAL MASSI
      COMMON /IRATE_TABLES/ VSNOWI(MDImin:MDImax)
      COMMON /IVENT_TABLES/ VENTI1(MDImin:MDImax), VENTI2(MDImin:MDImax)
!
!------------------------------------------------------------------------- 
!
!--- VEL_RF - velocity increase of rimed particles as functions of crude
!      particle size categories (at 0.1 mm intervals of mean ice particle
!      sizes) and rime factor (different values of Rime Factor of 1.1**N, 
!      where N=0 to Nrime).
!
      INTEGER, PARAMETER :: Nrime=40
      COMMON /IRIME_TABLES/ VEL_RF(2:9,0:Nrime)
!
!------------------------------------------------------------------------- 
!
!--- Mean rain drop diameters varying from 50 microns (0.05 mm) to 450 microns 
!      (0.45 mm), assuming an exponential size distribution.  
!
      REAL, PARAMETER :: DMRmin=.05e-3, DMRmax=.45e-3, DelDMR=1.e-6,
     & XMRmin=1.e6*DMRmin, XMRmax=1.e6*DMRmax
      INTEGER, PARAMETER :: MDRmin=XMRmin, MDRmax=XMRmax
!
      COMMON /RACCR_TABLES/ ACCRR(MDRmin:MDRmax)
      COMMON /RMASS_TABLES/ MASSR(MDRmin:MDRmax)
      REAL MASSR
      COMMON /RRATE_TABLES/ RRATE(MDRmin:MDRmax)
      COMMON /RVELR_TABLES/ VRAIN(MDRmin:MDRmax)
      COMMON /RVENT_TABLES/ VENTR1(MDRmin:MDRmax), VENTR2(MDRmin:MDRmax)
!
!------------------------------------------------------------------------- 
!------- Key parameters, local variables, & important comments ---------
!-----------------------------------------------------------------------
!
!--- KEY Parameters:
!
!---- Comments on 2 August 2000
!    * EPSQ=1.E-20 is the lower limit for specific humidity and cloud 
!      condensate.  The value of EPSQ will need to be changed in the other 
!      subroutines in order to make it consistent throughout the Eta code.  
!- NLImax - maximum number concentration of large ice crystals (20,000 /m**3, 20 per liter)
!- NLImin - minimum number concentration of large ice crystals (100 /m**3, 0.1 per liter)
!
      REAL, PARAMETER :: CP=1004.6, EPSQ=1.E-20, EPSW=1.E-12, RD=287.04, 
     & RHOL=1000., RV=461.5, T0C=273.15, XLS=2.834E6, EPS=RD/RV, 
     & NLImax=20.E3, NLImin=100., T_ICE=-10., T_ICE_init=-5., 
     & TOLER=5.E-7, 
     & CLIMIT=10.*EPSQ, CLIMIT1=-CLIMIT, EPS1=RV/RD-1., RCP=1./CP, 
     & RCPRV=RCP/RV, RRHOL=1./RHOL, XLS1=XLS*RCP, XLS2=XLS*XLS*RCPRV, 
     & XLS3=XLS*XLS/RV,
     & C1=1./3., C2=1./6., C3=3.31/6., 
     & DMR1=.1E-3, DMR2=.2E-3, DMR3=.32E-3, N0r0=8.E6, N0rmin=1.e4, 
     & N0s0=4.E6, RHO0=1.194, XMR1=1.e6*DMR1, XMR2=1.e6*DMR2, 
     & XMR3=1.e6*DMR3, Xratio=.025
      INTEGER, PARAMETER :: MDR1=XMR1, MDR2=XMR2, MDR3=XMR3
!
!--- If BLEND=1:
!      precipitation (large) ice amounts are estimated at each level as a 
!      blend of ice falling from the grid point above and the precip ice
!      present at the start of the time step (see TOT_ICE below).
!--- If BLEND=0:
!      precipitation (large) ice amounts are estimated to be the precip
!      ice present at the start of the time step.
!
!--- Extended to include sedimentation of rain on 2/5/01 
!
      REAL, PARAMETER :: BLEND=1.
!
!--- This variable is for debugging purposes (if .true.)
!
      LOGICAL, PARAMETER :: PRINT_diag=.TRUE.
!
!--- Local variables
!
      REAL EMAIRI
      LOGICAL DBG_logical, DIAG_print
!
!--- Full fields for vectorization
!
      LOGICAL :: ice_logical(myis:myie, myjs:myje)
      LOGICAL :: rain_logical(myis:myie, myjs:myje)
      LOGICAL :: toJump(myis:myie, myjs:myje, lm)
      LOGICAL :: clear(myis:myie, myjs:myje, lm)
      REAL    :: tc    (myis:myie, myjs:myje, lm)
      REAL    :: wv    (myis:myie, myjs:myje, lm)
      REAL    :: qsw   (myis:myie, myjs:myje, lm)
      REAL    :: qsi   (myis:myie, myjs:myje, lm)
      REAL    :: qswgrd(myis:myie, myjs:myje, lm)
      REAL    :: qsigrd(myis:myie, myjs:myje, lm)
      REAL    :: rho   (myis:myie, myjs:myje, lm)
      REAL    :: rrho  (myis:myie, myjs:myje, lm)
      REAL    :: bldtrh(myis:myie, myjs:myje, lm)
      REAL    :: arainnew(myis:myie, myjs:myje)
      REAL    :: asnownew(myis:myie, myjs:myje)
      REAL    :: qinew   (myis:myie, myjs:myje)
      REAL    :: qrnew   (myis:myie, myjs:myje)
      REAL    :: qwnew   (myis:myie, myjs:myje)
      REAL    :: pcond   (myis:myie, myjs:myje)
      REAL    :: pidep   (myis:myie, myjs:myje)
      REAL    :: piacw   (myis:myie, myjs:myje)
      REAL    :: piacwi  (myis:myie, myjs:myje)
      REAL    :: piacwr  (myis:myie, myjs:myje)
      REAL    :: piacr   (myis:myie, myjs:myje)
      REAL    :: picnd   (myis:myie, myjs:myje)
      REAL    :: pievp   (myis:myie, myjs:myje)
      REAL    :: pimlt   (myis:myie, myjs:myje)
      REAL    :: praut   (myis:myie, myjs:myje)
      REAL    :: pracw   (myis:myie, myjs:myje)
      REAL    :: prevp   (myis:myie, myjs:myje)
      REAL    :: xlv     (myis:myie, myjs:myje)
      REAL    :: xlv1    (myis:myie, myjs:myje)
      REAL    :: xlv2    (myis:myie, myjs:myje)
      REAL    :: xlf     (myis:myie, myjs:myje)
      REAL    :: xlf1    (myis:myie, myjs:myje)
      REAL    :: tk2     (myis:myie, myjs:myje)
      REAL    :: denomw  (myis:myie, myjs:myje)
      REAL    :: dynvis  (myis:myie, myjs:myje)
      REAL    :: therm_cond  (myis:myie, myjs:myje)
      REAL    :: diffus  (myis:myie, myjs:myje)
      REAL    :: gammas  (myis:myie, myjs:myje)
      REAL    :: gammar  (myis:myie, myjs:myje)
      REAL    :: qlice   (myis:myie, myjs:myje)
      REAL    :: qtice   (myis:myie, myjs:myje)
      REAL    :: wvqw    (myis:myie, myjs:myje)
      REAL    :: fsmall  (myis:myie, myjs:myje)
      REAL    :: xsimass (myis:myie, myjs:myje)
      INTEGER :: indexs  (myis:myie, myjs:myje)
      REAL    :: tot_ice (myis:myie, myjs:myje)
      REAL    :: piloss  (myis:myie, myjs:myje)
      REAL    :: rimef1  (myis:myie, myjs:myje)
      REAL    :: vel_inc (myis:myie, myjs:myje)
      REAL    :: vsnow   (myis:myie, myjs:myje)
      REAL    :: flimass (myis:myie, myjs:myje)
      REAL    :: nlice   (myis:myie, myjs:myje)
      REAL    :: nsmice  (myis:myie, myjs:myje)
      REAL    :: dwvi    (myis:myie, myjs:myje)
      REAL    :: dum     (myis:myie, myjs:myje)
      REAL    :: dum1    (myis:myie, myjs:myje)
      REAL    :: dum2    (myis:myie, myjs:myje)
      REAL    :: vrimef  (myis:myie, myjs:myje)
      INTEGER :: ixs     (myis:myie, myjs:myje)
      REAL    :: pidep_max(myis:myie, myjs:myje)
      REAL    :: ventil  (myis:myie, myjs:myje)
      REAL    :: tcc     (myis:myie, myjs:myje)
      REAL    :: tot_rain(myis:myie, myjs:myje)
      REAL    :: tot_rainnew(myis:myie, myjs:myje)
      REAL    :: vrain1     (myis:myie, myjs:myje)
      REAL    :: qtrain     (myis:myie, myjs:myje)
      REAL    :: prloss     (myis:myie, myjs:myje)
      REAL    :: rqr        (myis:myie, myjs:myje)
      REAL    :: n0r        (myis:myie, myjs:myje)
      INTEGER :: indexr     (myis:myie, myjs:myje)
      REAL    :: rr         (myis:myie, myjs:myje)
      REAL    :: dwvr       (myis:myie, myjs:myje)
      REAL    :: tnew       (myis:myie, myjs:myje)
      REAL    :: wvnew      (myis:myie, myjs:myje)
      REAL    :: rimef      (myis:myie, myjs:myje)
      REAL    :: tot_icenew (myis:myie, myjs:myje)
      REAL    :: vrain2     (myis:myie, myjs:myje)
      REAL    :: wcnew      (myis:myie, myjs:myje)
!
!
!#######################################################################
!########################## Begin Execution ############################
!#######################################################################
!
!
!----------------------------------------------------------------------
!------------ Vertical levels to use ----------------------------------
!--- Skip level if inside terrain or 
!      no condensate and very low specific humidities
!
      DO l=1,lm
         DO j=myjs2, myje2
            DO i=myis,  myie
               toJump(i,j,l) = (l > lmh(i,j)) .OR.
     &              (qv_col(i,j,l).LE.EPSQ .AND. wc_col(i,j,l).LE.EPSQ)
               IF (.NOT. toJump(i,j,l)) THEN
                  TC(I,J,L)=T_COL(I,J,L)-T0C ! Temperature (deg C)
                  WV(I,J,L)=QV_COL(I,J,L)/(1.-QV_COL(I,J,L)) ! Water vapor mixing ratio (kg/kg)
!
!-----------------------------------------------------------------------
!--- Moisture variables below are mixing ratios & not specifc humidities
!-----------------------------------------------------------------------
!
!    
!--- This check is to determine grid-scale saturation when no condensate is present
!    
                  ESW=1000.*FPVS0(T_COL(I,J,L)) ! Saturation vapor pressure w/r/t water
                  QSW(I,J,L)=EPS*ESW/(P_COL(I,J,L)-ESW) ! Saturation mixing ratio w/r/t water
                  IF (TC(I,J,L) .LT. 0.) THEN
                     ESI=1000.*FPVS(T_COL(I,J,L)) ! Saturation vapor pressure w/r/t ice
                     QSI(I,J,L)=EPS*ESI/(P_COL(I,J,L)-ESI) ! Saturation mixing ratio w/r/t water
                     WS=QSI(I,J,L) ! General saturation mixing ratio (water/ice)
                  ELSE
                     WS=QSW(I,J,L) ! General saturation mixing ratio (water/ice)
                  ENDIF
!
!--- Effective grid-scale Saturation mixing ratios
!
                  Qswgrd(I,J,L)=RHgrd*QSW(I,J,L)
                  Qsigrd(I,J,L)=RHgrd*QSI(I,J,L)
!
!--- Check if air is subsaturated and w/o condensate
!
                  CLEAR(I,J,L) = .NOT. (
     &                 WV(I,J,L).GT.RHgrd*WS .OR. 
     &                 WC_COL(I,J,L).GT.EPSW) 
               END IF
            END DO
         END DO
      END DO

      DO j=myjs2, myje2
         DO i=myis,  myie
            ARAIN(I,J)=0.       ! Accumulated rainfall into grid box from above (kg/m**2)
            ASNOW(I,J)=0.       ! Accumulated snowfall into grid box from above (kg/m**2)
         END DO
      END DO

      DIAG_print=.false.
      DO L=1,LM
         DO j=myjs2, myje2
            DO i=myis,  myie
!
!-----------------------------------------------------------------------
!------------ Loop from top (L=1) to surface (L=LSFC) ------------------
!-----------------------------------------------------------------------
!
!
!--- Skip this level?
!
               IF (toJump(i,j,l)) CYCLE
!
!-----------------------------------------------------------------------
!------------ Proceed with cloud microphysics calculations -------------
!-----------------------------------------------------------------------
!
!
!--- Check if any rain is falling into layer from above
!
               IF (ARAIN(I,J) .GT. CLIMIT) THEN
                  CLEAR(I,J,L)=.FALSE.
               ELSE
                  ARAIN(I,J)=0.
               ENDIF
!
!--- Check if any ice is falling into layer from above
!
!--- NOTE that "SNOW" in variable names is synonomous with 
!    large, precipitation ice particles
!
               IF (ASNOW(I,J) .GT. CLIMIT) THEN
                  CLEAR(I,J,L)=.FALSE.
               ELSE
                  ASNOW(I,J)=0.
               ENDIF

               toJump(i,j,l) = toJump(i,j,l) .AND. CLEAR(i,j,l)
            END DO
         END DO
!
!-----------------------------------------------------------------------
!-- Loop to the end if in clear, subsaturated air free of condensate ---
!-----------------------------------------------------------------------
!
         DO j=myjs2, myje2
            DO i=myis,  myie
               IF (toJump(i,j,l)) CYCLE
!
!-----------------------------------------------------------------------
!--------- Initialize RHO, THICK & microphysical processes -------------
!-----------------------------------------------------------------------
!
!
!--- Virtual temperature, TV=T*(1./EPS-1)*Q, Q is specific humidity;
!    (see pp. 63-65 in Fleagle & Businger, 1963)
!
               RHO(I,J,L)=P_COL(I,J,L)/
     &              (RD*T_COL(I,J,L)*(1.+EPS1*QV_COL(I,J,L))) ! Air density (kg/m**3)
               RRHO(I,J,L)=1./RHO(I,J,L) ! Reciprocal of air density
               BLDTRH(I,J,L)=BLEND*DTPH*RHO(I,J,L) ! Blend parameter * time step * air density
!
               Arainnew(I,J)=0. ! Updated accumulated rainfall
               Asnownew(I,J)=0. ! Updated accumulated snowfall
               Qinew(I,J)=0.    ! Updated ice mixing ratio
               Qrnew(I,J)=0.    ! Updated rain ratio
               Qwnew(I,J)=0.    ! Updated cloud water ratio
!
               PCOND(I,J)=0.    ! Condensation (>0) or evaporation (<0) of cloud water (kg/kg)
               PIDEP(I,J)=0.    ! Deposition (>0) or sublimation (<0) of ice crystals (kg/kg)
               PIACW(I,J)=0.    ! Cloud water collection (riming) by precipitation ice (kg/kg; >0)
               PIACWI(I,J)=0.   ! Growth of precip ice by riming (kg/kg; >0)
               PIACWR(I,J)=0.   ! Shedding of accreted cloud water to form rain (kg/kg; >0)
               PIACR(I,J)=0.    ! Freezing of rain onto large ice at supercooled temps (kg/kg; >0)
               PICND(I,J)=0.    ! Condensation (>0) onto wet, melting ice (kg/kg)
               PIEVP(I,J)=0.    ! Evaporation (<0) from wet, melting ice (kg/kg)
               PIMLT(I,J)=0.    ! Melting ice (kg/kg; >0)
               PRAUT(I,J)=0.    ! Cloud water autoconversion to rain (kg/kg; >0)
               PRACW(I,J)=0.    ! Cloud water collection (accretion) by rain (kg/kg; >0)
               PREVP(I,J)=0.    ! Rain evaporation (kg/kg; <0)
!
!--- Double check input hydrometeor mixing ratios
!
!          DUM=WC_COL(I,J,L)-(QI_COL(I,J,L)+QW_COL(I,J,L)+QR_COL(I,J,L))
!          DUM1=ABS(DUM)
!          DUM2=TOLER*MIN(WC_COL(I,J,L), QI_COL(I,J,L)+QW_COL(I,J,L)+QR)
!
!***********************************************************************
!*********** MAIN MICROPHYSICS CALCULATIONS NOW FOLLOW! ****************
!***********************************************************************
!
!--- Calculate a few variables, which are used more than once below
!
!--- Latent heat of vaporization as a function of temperature from
!      Bolton (1980, JAS)
!
               XLV(I,J)=3.148E6-2370*T_COL(I,J,L) ! Latent heat of vaporization (Lv)
               XLF(I,J)=XLS-XLV(I,J) ! Latent heat of fusion (Lf)
               XLV1(I,J)=XLV(I,J)*RCP ! Lv/Cp
               XLF1(I,J)=XLF(I,J)*RCP ! Lf/Cp
               TK2(I,J)=1./(T_COL(I,J,L)*T_COL(I,J,L)) ! 1./TK**2
               XLV2(I,J)=XLV(I,J)*XLV(I,J)*QSW(I,J,L)*TK2(I,J)/RV ! Lv**2*Qsw/(Rv*TK**2)
               DENOMW(I,J)=1.+XLV2(I,J)*RCP ! Denominator term, Clausius-Clapeyron correction
!
!--- Basic thermodynamic quantities
!      * DYNVIS - dynamic viscosity  [ kg/(m*s) ]
!      * THERM_COND - thermal conductivity  [ J/(m*s*K) ]
!      * DIFFUS - diffusivity of water vapor  [ m**2/s ]
!
               TFACTOR=T_COL(I,J,L)**1.5/(T_COL(I,J,L)+120.)
               DYNVIS(I,J)=1.496E-6*TFACTOR
               THERM_COND(I,J)=2.116E-3*TFACTOR
               DIFFUS(I,J)=8.794E-5*T_COL(I,J,L)**1.81/P_COL(I,J,L)
!
!--- Air resistance term for the fall speed of ice following the
!      basic research by Heymsfield, Kajikawa, others 
!
               GAMMAS(I,J)=(1.E5/P_COL(I,J,L))**C1
!
!--- Air resistance for rain fall speed (Beard, 1985, JAS, p.470)
!
               GAMMAR(I,J)=(RHO0/RHO(I,J,L))**.4
!
!----------------------------------------------------------------------
!-------------  IMPORTANT MICROPHYSICS DECISION TREE  -----------------
!----------------------------------------------------------------------
!
!--- Determine if conditions supporting ice are present
!
               IF (TC(I,J,L).LT.0. .OR. 
     &              QI_COL(I,J,L).GT. CLIMIT .OR. 
     &              ASNOW(I,J).GT.CLIMIT) THEN
                  ice_logical(i,j)=.TRUE.
               ELSE
                  ice_logical(i,j)=.FALSE.
                  QLICE(I,J)=0.
                  QTICE(I,J)=0.
               ENDIF
!
!--- Determine if rain is present
!
               rain_logical(i,j)= (ARAIN(I,J).GT.CLIMIT .OR. 
     &              QR_COL(I,J,L).GT.CLIMIT)
            END DO
         END DO

         DO j=myjs2, myje2
            DO i=myis,  myie
               IF (toJump(i,j,l)) CYCLE

!
               IF (ice_logical(i,j)) THEN
!
!--- IMPORTANT:  Estimate time-averaged properties.
!
!---
!  * FLARGE  - ratio of number of large ice to total (large & small) ice
!  * FSMALL  - ratio of number of small ice crystals to large ice particles
!  ->  Small ice particles are assumed to have a mean diameter of 50 microns.
!  * XSIMASS - used for calculating small ice mixing ratio
!---
!  * TOT_ICE - total mass (small & large) ice before microphysics,
!              which is the sum of the total mass of large ice in the 
!              current layer and the input flux of ice from above
!  * PILOSS  - greatest loss (<0) of total (small & large) ice by
!              sublimation, removing all of the ice falling from above
!              and the ice within the layer
!  * RimeF1  - Rime Factor, which is the mass ratio of total (unrimed & rimed) 
!              ice mass to the unrimed ice mass (>=1)
!  * VrimeF  - the velocity increase due to rime factor or melting (ratio, >=1)
!  * VSNOW   - Fall speed of rimed snow w/ air resistance correction
!  * EMAIRI  - equivalent mass of air associated layer and with fall of snow into layer
!  * XLIMASS - used for calculating large ice mixing ratio
!  * FLIMASS - mass fraction of large ice
!  * QTICE   - time-averaged mixing ratio of total ice
!  * QLICE   - time-averaged mixing ratio of large ice
!  * NLICE   - time-averaged number concentration of large ice
!  * NSmICE  - number concentration of small ice crystals at current level
!---
!--- Assumed number fraction of large ice particles to total (large & small) 
!    ice particles, which is based on a general impression of the literature.
!
                  WVQW(I,J)=WV(I,J,L)+QW_COL(I,J,L) ! Water vapor & cloud water
!
                  IF (TC(I,J,L).GE.0. .OR. 
     &                 WVQW(I,J).LT.Qsigrd(I,J,L)) THEN
   !
   !--- Eliminate small ice particle contributions for melting & sublimation
   !
                     FLARGE=1.
                  ELSE
   !
   !--- Enhanced number of small ice particles during depositional growth
   !    (effective only when 0C > T >= T_ice [-10C] )
   !
                     FLARGE=.2
   !
   !--- Larger number of small ice particles due to rime splintering
   !
                     IF (TC(I,J,L).GE.-8. .AND. TC(I,J,L).LE.-3.) 
     &                    FLARGE=.5*FLARGE
!
                  ENDIF         ! End IF (TC(I,J,L).GE.0. .OR. WVQW.LT.QSIgrd)
                  FSMALL(I,J)=(1.-FLARGE)/FLARGE
                  XSIMASS(I,J)=RRHO(I,J,L)*MASSI(MDImin)*FSMALL(I,J)
                  IF (QI_COL(I,J,L).LE.CLIMIT .AND. 
     &                 ASNOW(I,J).LE.CLIMIT) THEN
              INDEXS(I,J)=MDImin
              TOT_ICE(I,J)=0.
              PILOSS(I,J)=0.
              Rimef1(I,J)=1.
              Vrimef(I,J)=1.
              VEL_INC(I,J)=GAMMAS(I,J)
              VSNOW(I,J)=0.
              XLIMASS=RRHO(I,J,L)*Rimef1(I,J)*MASSI(INDEXS(I,J))
              FLIMASS(I,J)=XLIMASS/(XLIMASS+XSIMASS(I,J))
              QLICE(I,J)=0.
              QTICE(I,J)=0.
              NLICE(I,J)=0.
              Nsmice(I,J)=0.
            ELSE
   !
   !--- For T<0C mean particle size follows Houze et al. (JAS, 1979, p. 160), 
   !    converted from Fig. 5 plot of LAMDAs.  Similar set of relationships 
   !    also shown in Fig. 8 of Ryan (BAMS, 1996, p. 66).
   !
              DUM(I,J)=XMImax*EXP(.0536*TC(I,J,L))
              INDEXS(I,J)=MIN(MDImax, MAX(MDImin, INT(DUM(I,J)) ) )
              TOT_ICE(I,J)=THICK_COL(I,J,L)*QI_COL(I,J,L)+
     &             BLEND*ASNOW(I,J)
              PILOSS(I,J)=-TOT_ICE(I,J)/THICK_COL(I,J,L)
              LBEF=MAX(1,L-1)
              DUM1(I,J)=RimeF_col(i,j,LBEF)
              DUM2(I,J)=rimef_col(i,j,l)
              Rimef1(I,J)=(DUM2(I,J)*THICK_COL(I,J,L)*QI_COL(I,J,L)+
     &             DUM1(I,J)*BLEND*ASNOW(I,J))/TOT_ICE(I,J)
              Rimef1(I,J)=MIN(Rimef1(I,J), RFmax)
              DO IPASS=0,1
                IF (Rimef1(I,J) .LE. 1.) THEN
                  Rimef1(I,J)=1.
                  Vrimef(I,J)=1.
                ELSE
                  IXS(I,J)=MAX(2, MIN(INDEXS(I,J)/100, 9))
                  XRF=10.492*ALOG(Rimef1(I,J))
                  IXRF=MAX(0, MIN(INT(XRF), Nrime))
                  IF (IXRF .GE. Nrime) THEN
                    Vrimef(I,J)=VEL_RF(IXS(I,J),Nrime)
                  ELSE
                    Vrimef(I,J)=VEL_RF(IXS(I,J),IXRF)+
     &                    (XRF-FLOAT(IXRF))*
     &                    (VEL_RF(IXS(I,J),IXRF+1)-
     &                    VEL_RF(IXS(I,J),IXRF))
                  ENDIF
                ENDIF            ! End IF (Rimef1(I,J) .LE. 1.)
                VEL_INC(I,J)=GAMMAS(I,J)*Vrimef(I,J)
                VSNOW(I,J)=VEL_INC(I,J)*VSNOWI(INDEXS(I,J))
                EMAIRI=THICK_COL(I,J,L)+BLDTRH(I,J,L)*VSNOW(I,J)
                XLIMASS=RRHO(I,J,L)*Rimef1(I,J)*MASSI(INDEXS(I,J))
                FLIMASS(I,J)=XLIMASS/(XLIMASS+XSIMASS(I,J))
                QTICE(I,J)=TOT_ICE(I,J)/EMAIRI
                QLICE(I,J)=FLIMASS(I,J)*QTICE(I,J)
                NLICE(I,J)=QLICE(I,J)/XLIMASS
                Nsmice(I,J)=Fsmall(I,J)*NLICE(I,J)
   !
                IF ( (NLICE(I,J).GE.NLImin .AND. NLICE(I,J).LE.NLImax) 
     &                .OR. IPASS.EQ.1) THEN
                  EXIT
                ELSE
        !
        !--- Reduce excessive accumulation of ice at upper levels
        !    associated with strong grid-resolved ascent
        !
        !--- Force NLICE to be between NLImin and NLImax
        !
                  DUM(I,J)=MAX(NLImin, MIN(NLImax, NLICE(I,J)) )
                  XLI=RHO(I,J,L)*(QTICE(I,J)/DUM(I,J)-XSIMASS(I,J))/
     &                 Rimef1(I,J)
                  IF (XLI .LE. MASSI(MDImin) ) THEN
                    INDEXS(I,J)=MDImin
                  ELSE IF (XLI .LE. MASSI(450) ) THEN
                    DLI=9.5885E5*XLI**.42066         ! DLI in microns
                    INDEXS(I,J)=MIN(MDImax, MAX(MDImin, INT(DLI) ) )
                  ELSE IF (XLI .LE. MASSI(MDImax) ) THEN
                    DLI=3.9751E6*XLI**.49870         ! DLI in microns
                    INDEXS(I,J)=MIN(MDImax, MAX(MDImin, INT(DLI) ) )
                  ELSE 
                    INDEXS(I,J)=MDImax
           !
           !--- 8/22/01: Increase density of large ice if maximum limits 
           !    are reached for number concentration (NLImax) and mean size 
           !    (MDImax).  Done to increase fall out of ice.
           !
                    IF (DUM(I,J) .GE. NLImax) Rimef1(I,J)=RHO(I,J,L)*
     &                   (QTICE(I,J)/NLImax-XSIMASS(I,J))/
     &                   MASSI(INDEXS(I,J))
                  ENDIF             ! End IF (XLI .LE. MASSI(MDImin) ) 
!            WRITE(6,"(4(a12,g11.4,1x))") 
!     & '{$ TC=',TC(I,J,L),'P=',.01*P_COL(I,J,L),'NLICE=',NLICE,'DUM=',DUM,
!     & '{$ XLI=',XLI,'INDEXS=',FLOAT(INDEXS),'RHO=',RHO,'QTICE=',QTICE,
!     & '{$ XSIMASS=',XSIMASS,'RimeF1=',RimeF1
                ENDIF                  ! End IF ( (NLICE.GE.NLImin .AND. NLICE.LE.NLImax) ...
              ENDDO                    ! End DO IPASS=0,1
            ENDIF                      ! End IF (QI.LE.CLIMIT .AND. ASNOW(I,J).LE.CLIMIT)
          ENDIF                        ! End IF (ICE_logical)
!
!----------------------------------------------------------------------
!--------------- Calculate individual processes -----------------------
!----------------------------------------------------------------------
!
!--- Cloud water autoconversion to rain and collection by rain
!
          IF (QW_COL(I,J,L).GT.CLIMIT .AND. TC(I,J,L).GE.T_ICE) THEN
   !
   !--- QW0 could be modified based on land/sea properties, 
   !      presence of convection, etc.  This is why QAUT0 and CRAUT
   !      are passed into the subroutine as externally determined
   !      parameters.  Can be changed in the future if desired.
   !
            QW0=QAUT0*RRHO(I,J,L)
            PRAUT(I,J)=MAX(0., QW_COL(I,J,L)-QW0)*CRAUT
            IF (QLICE(I,J) .GT. CLIMIT) THEN
      !
      !--- Collection of cloud water by large ice particles ("snow")
      !    PIACWI=PIACW for riming, PIACWI=0 for shedding
      !
              FWS=MIN(1., 
     &              CIACW*VEL_INC(I,J)*NLICE(I,J)*
     &              ACCRI(INDEXS(I,J))/P_COL(I,J,L)**C1)
              PIACW(I,J)=FWS*QW_COL(I,J,L)
              IF (TC(I,J,L) .LT. 0.) PIACWI(I,J)=PIACW(I,J)    ! Large ice riming
            ENDIF           ! End IF (QLICE .GT. CLIMIT)
          ENDIF             ! End IF (QW.GT.CLIMIT .AND. TC.GE.T_ICE)
!
!----------------------------------------------------------------------
!--- Loop around some of the ice-phase processes if no ice should be present
!----------------------------------------------------------------------
!
          IF (ice_logical(i,j)) THEN
!
!--- Now the pretzel logic of calculating ice deposition
!
          IF (TC(I,J,L).LT.T_ICE .AND. 
     &         (WV(I,J,L).GT.Qsigrd(I,J,L) .OR. 
     &         QW_COL(I,J,L).GT.CLIMIT)) THEN
   !
   !--- Adjust to ice saturation at T<T_ICE (-10C) if supersaturated.
   !    Sources of ice due to nucleation and convective detrainment are
   !    either poorly understood, poorly resolved at typical NWP 
   !    resolutions, or are not represented (e.g., no detrained 
   !    condensate in BMJ Cu scheme).
   !    
            PCOND(I,J)=-QW_COL(I,J,L)
            DUM1(I,J)=T_COL(I,J,L)+XLV1(I,J)*PCOND(I,J)                 ! Updated (dummy) temperature (deg K)
            DUM2(I,J)=WV(I,J,L)+QW_COL(I,J,L)                         ! Updated (dummy) water vapor mixing ratio
            DUM(I,J)=1000.*FPVS(DUM1(I,J))               ! Updated (dummy) saturation vapor pressure w/r/t ice
            DUM(I,J)=RHgrd*EPS*DUM(I,J)/(P_COL(I,J,L)-DUM(I,J))         ! Updated (dummy) saturation mixing ratio w/r/t ice
            IF (DUM2(I,J) .GT. DUM(I,J)) 
     &           PIDEP(I,J)=DEPOSIT (P_COL(I,J,L), RHgrd, 
     &           DUM1(I,J), DUM2(I,J))
            Dwvi(I,J)=0.    ! Used only for debugging
   !
          ELSE IF (TC(I,J,L) .LT. 0.) THEN
   !
   !--- These quantities are handy for ice deposition/sublimation
   !    PIDEP_max - max deposition or minimum sublimation to ice saturation
   !
            DENOMI=1.+XLS2*QSI(I,J,L)*TK2(I,J)
            Dwvi(I,J)=MIN(WVQW(I,J),QSW(I,J,L))-QSI(I,J,L)
            pidep_max(i,j)=MAX(PILOSS(I,J), Dwvi(I,J)/DENOMI)
            IF (QTICE(I,J) .GT. 0.) THEN
      !
      !--- Calculate ice deposition/sublimation
      !      * SFACTOR - [VEL_INC**.5]*[Schmidt**(1./3.)]*[(RHO/DYNVIS)**.5],
      !        where Schmidt (Schmidt Number) =DYNVIS/(RHO*DIFFUS)
      !      * Units: SFACTOR - s**.5/m ;  ABI - m**2/s ;  NLICE - m**-3 ;
      !               VENTIL, VENTIS - m**-2 ;  VENTI1 - m ;  
      !               VENTI2 - m**2/s**.5 ; DIDEP - unitless
      !
              SFACTOR=VEL_INC(I,J)**.5*(RHO(I,J,L)/
     &              (DIFFUS(I,J)*DIFFUS(I,J)*DYNVIS(i,j)))**C2
              ABI=1./(RHO(I,J,L)*XLS3*QSI(I,J,L)*
     &             TK2(I,J)/THERM_COND(I,J)+1./DIFFUS(I,J))
      !
      !--- VENTIL - Number concentration * ventilation factors for large ice
      !--- VENTIS - Number concentration * ventilation factors for small ice
      !
      !--- Variation in the number concentration of ice with time is not
      !      accounted for in these calculations (could be in the future).
      !
              VENTIL(I,J)=(VENTI1(INDEXS(I,J))+
     &             SFACTOR*VENTI2(INDEXS(I,J)))*NLICE(I,J)
              VENTIS=(VENTI1(MDImin)+SFACTOR*VENTI2(MDImin))*Nsmice(I,J)
              DIDEP=ABI*(VENTIL(I,J)+VENTIS)*DTPH
      !
      !--- Account for change in water vapor supply w/ time
      !
              IF (DIDEP .GE. Xratio) 
     &          DIDEP=(1.-EXP(-DIDEP*DENOMI))/DENOMI
              IF (Dwvi(I,J) .GT. 0.) THEN
                PIDEP(I,J)=MIN(Dwvi(I,J)*DIDEP, pidep_max(i,j))
              ELSE IF (Dwvi(I,J) .LT. 0.) THEN
                PIDEP(I,J)=MAX(Dwvi(I,J)*DIDEP, pidep_max(i,j))
              ENDIF
      !
            ELSE IF (WVQW(I,J).GT.QSI(I,J,L) .AND. 
     &             TC(I,J,L).LE.T_ICE_init) THEN
      !
      !--- Ice nucleation in near water-saturated conditions.  Ice crystal
      !    growth during time step calculated using Miller & Young (1979, JAS).
      !--- These deposition rates could drive conditions below water saturation,
      !    which is the basis of these calculations.  Intended to approximate
      !    more complex & computationally intensive calculations.
      !
              INDEX_MY=MAX(MY_T1, MIN( INT(.5-TC(I,J,L)), MY_T2 ) )
      !
      !--- DUM1 is the supersaturation w/r/t ice at water-saturated conditions
      !
      !--- DUM2 is the number of ice crystals nucleated at water-saturated 
      !    conditions based on Meyers et al. (JAM, 1992).
      !
      !--- Prevent unrealistically large ice initiation (limited by PIDEP_max)
      !      if DUM2(I,J) values are increased in future experiments
      !
              DUM1(I,J)=QSW(I,J,L)/QSI(I,J,L)-1.      
              DUM2(I,J)=1.E3*EXP(12.96*DUM1(I,J)-.039)
              PIDEP(I,J)=MIN(pidep_max(i,j), 
     &             DUM2(I,J)*MY_GROWTH(INDEX_MY)*RRHO(I,J,L))
      !
            ENDIF       ! End IF (QTICE .GT. 0.)
   !
          ENDIF         ! End IF (TC.LT.T_ICE .AND. (WV.GT.QSIgrd .OR. QW.GT.CLIMIT))
!
!------------------------------------------------------------------------
!
          END IF     ! ICE_logical
!
!------------------------------------------------------------------------
!
!--- Cloud water condensation
!
          IF (TC(I,J,L).GE.T_ICE .AND. 
     &       (QW_COL(I,J,L).GT.CLIMIT .OR. 
     &       WV(I,J,L).GT.Qswgrd(I,J,L))) THEN
            IF (PIACWI(I,J).EQ.0. .AND. PIDEP(I,J).EQ.0.) THEN
              PCOND(I,J)=CONDENSE (P_COL(I,J,L), QW_COL(I,J,L), 
     &              RHgrd, T_COL(I,J,L), WV(I,J,L))
            ELSE
   !
   !--- Modify cloud condensation in response to ice processes
   !
              DUM(I,J)=XLV(I,J)*Qswgrd(I,J,L)*RCPRV*TK2(I,J)
              DENOMWI=1.+XLS*DUM(I,J)
              DENOMF=XLF(I,J)*DUM(I,J)
              DUM(I,J)=MAX(0., PIDEP(I,J))
              PCOND(I,J)=(WV(I,J,L)-Qswgrd(I,J,L)-
     &             DENOMWI*DUM(I,J)-DENOMF*PIACWI(I,J))/DENOMW(I,J)
              DUM1(I,J)=-QW_COL(I,J,L)
              DUM2(I,J)=PCOND(I,J)-PIACW(I,J)
              IF (DUM2(I,J) .LT. DUM1(I,J)) THEN
      !
      !--- Limit cloud water sinks
      !
                DUM(I,J)=DUM1(I,J)/DUM2(I,J)
                PCOND(I,J)=DUM(I,J)*PCOND(I,J)
                PIACW(I,J)=DUM(I,J)*PIACW(I,J)
                PIACWI(I,J)=DUM(I,J)*PIACWI(I,J)
              ENDIF        ! End IF (DUM2 .LT. DUM1)
            ENDIF          ! End IF (PIACWI.EQ.0. .AND. PIDEP.EQ.0.)
          ENDIF            ! End IF (TC.GE.T_ICE .AND. (QW.GT.CLIMIT .OR. WV.GT.QSWgrd))
!
!--- Limit freezing of accreted rime to prevent temperature oscillations,
!    a crude Schumann-Ludlam limit (p. 209 of Young, 1993). 
!
          TCC(I,J)=TC(I,J,L)+XLV1(I,J)*PCOND(I,J)+XLS1*PIDEP(I,J)+
     &         XLF1(I,J)*PIACWI(I,J)
          IF (TCC(I,J) .GT. 0.) THEN
            PIACWI(I,J)=0.
            TCC(I,J)=TC(I,J,L)+XLV1(I,J)*PCOND(I,J)+XLS1*PIDEP(I,J)
          ENDIF
!
          IF (TC(I,J,L).GT.0. .AND. 
     &         TCC(I,J).GT.0. .AND. 
     &         ice_logical(i,j)) THEN
   !
   !--- Calculate melting and evaporation/condensation
   !      * Units: SFACTOR - s**.5/m ;  ABI - m**2/s ;  NLICE - m**-3 ;
   !               VENTIL - m**-2 ;  VENTI1 - m ;  
   !               VENTI2 - m**2/s**.5 ; CIEVP - /s
   !
            SFACTOR=VEL_INC(I,J)**.5*
     &            (RHO(I,J,L)/(DIFFUS(I,J)*DIFFUS(I,J)*DYNVIS(i,j)))**C2
            VENTIL(I,J)=NLICE(I,J)*(VENTI1(INDEXS(I,J))+
     &           SFACTOR*VENTI2(INDEXS(I,J)))
            AIEVP=VENTIL(I,J)*DIFFUS(I,J)*DTPH
            IF (AIEVP .LT. Xratio) THEN
              DIEVP=AIEVP
            ELSE
              DIEVP=1.-EXP(-AIEVP)
            ENDIF
            QSW0=EPS*ESW0/(P_COL(I,J,L)-ESW0)
            DWV0=MIN(WV(I,J,L),QSW(I,J,L))-QSW0
            DUM(I,J)=QW_COL(I,J,L)+PCOND(I,J)
            IF (WV(I,J,L).LT.QSW(I,J,L) .AND. DUM(I,J).LE.CLIMIT) THEN
   !
   !--- Evaporation from melting snow (sink of snow) or shedding
   !    of water condensed onto melting snow (source of rain)
   !
              DUM(I,J)=DWV0*DIEVP
              PIEVP(I,J)=MAX( MIN(0., DUM(I,J)), PILOSS(I,J))
              PICND(I,J)=MAX(0., DUM(I,J))
            ENDIF            ! End IF (WV.LT.QSW .AND. DUM.LE.CLIMIT)
            PIMLT(I,J)=THERM_COND(I,J)*TCC(I,J)*VENTIL(I,J)*
     &           RRHO(I,J,L)*DTPH/XLF(I,J)
   !
   !--- Limit melting to prevent temperature oscillations across 0C
   !
            DUM1(I,J)=MAX( 0., 
     &           (TCC(I,J)+XLV1(I,J)*PIEVP(I,J))/XLF1(I,J) )
            PIMLT(I,J)=MIN(PIMLT(I,J), DUM1(I,J))
   !
   !--- Limit loss of snow by melting (>0) and evaporation
   !
            DUM(I,J)=PIEVP(I,J)-PIMLT(I,J)
            IF (DUM(I,J) .LT. PILOSS(I,J)) THEN
              DUM1(I,J)=PILOSS(I,J)/DUM(I,J)
              PIMLT(I,J)=PIMLT(I,J)*DUM1(I,J)
              PIEVP(I,J)=PIEVP(I,J)*DUM1(I,J)
            ENDIF           ! End IF (DUM .GT. QTICE)
          ENDIF             ! End IF (TC.GT.0. .AND. TCC.GT.0. .AND. ICE_logical) 
!
!--- IMPORTANT:  Estimate time-averaged properties.
!
!  * TOT_RAIN - total mass of rain before microphysics, which is the sum of
!               the total mass of rain in the current layer and the input 
!               flux of rain from above
!  * VRAIN1   - fall speed of rain into grid from above (with air resistance correction)
!  * QTRAIN   - time-averaged mixing ratio of rain (kg/kg)
!  * PRLOSS   - greatest loss (<0) of rain, removing all rain falling from
!               above and the rain within the layer
!  * RQR      - rain content (kg/m**3)
!  * INDEXR   - mean size of rain drops to the nearest 1 micron in size
!  * N0r      - intercept of rain size distribution (typically 10**6 m**-4)
!
          TOT_RAIN(I,J)=0.
          VRAIN1(I,J)=0.
          QTRAIN(I,J)=0.
          PRLOSS(I,J)=0.
          RQR(I,J)=0.
          N0r(I,J)=0.
          INDEXR(I,J)=MDRmin
          IF (rain_logical(i,j)) THEN
            IF (ARAIN(I,J) .LE. 0.) THEN
              INDEXR(I,J)=MDRmin
              VRAIN1(I,J)=0.
            ELSE
   !
   !--- INDEXR (related to mean diameter) & N0r could be modified 
   !      by land/sea properties, presence of convection, etc.
   !
   !--- Rain rate normalized to a density of 1.194 kg/m**3
   !
              RR(I,J)=ARAIN(I,J)/(DTPH*GAMMAR(I,J))
   !
              IF (RR(I,J) .LE. RR_DRmin) THEN
        !
        !--- Assume fixed mean diameter of rain (0.2 mm) for low rain rates, 
        !      instead vary N0r with rain rate
        !
                INDEXR(I,J)=MDRmin
              ELSE IF (RR(I,J) .LE. RR_DR1) THEN
        !
        !--- Best fit to mass-weighted fall speeds (V) from rain lookup tables 
        !      for mean diameters (Dr) between 0.05 and 0.10 mm:
        !      V(Dr)=5.6023e4*Dr**1.136, V in m/s and Dr in m
        !      RR = PI*1000.*N0r0*5.6023e4*Dr**(4+1.136) = 1.408e15*Dr**5.136,
        !        RR in kg/(m**2*s)
        !      Dr (m) = 1.123e-3*RR**.1947 -> Dr (microns) = 1.123e3*RR**.1947
        !
                INDEXR(I,J)=INT( 1.123E3*RR(I,J)**.1947 + .5 )
                INDEXR(I,J)=MAX( MDRmin, MIN(INDEXR(I,J), MDR1) )
              ELSE IF (RR(I,J) .LE. RR_DR2) THEN
        !
        !--- Best fit to mass-weighted fall speeds (V) from rain lookup tables 
        !      for mean diameters (Dr) between 0.10 and 0.20 mm:
        !      V(Dr)=1.0867e4*Dr**.958, V in m/s and Dr in m
        !      RR = PI*1000.*N0r0*1.0867e4*Dr**(4+.958) = 2.731e14*Dr**4.958,
        !        RR in kg/(m**2*s)
        !      Dr (m) = 1.225e-3*RR**.2017 -> Dr (microns) = 1.225e3*RR**.2017
        !
                INDEXR(I,J)=INT( 1.225E3*RR(I,J)**.2017 + .5 )
                INDEXR(I,J)=MAX( MDR1, MIN(INDEXR(I,J), MDR2) )
              ELSE IF (RR(I,J) .LE. RR_DR3) THEN
        !
        !--- Best fit to mass-weighted fall speeds (V) from rain lookup tables 
        !      for mean diameters (Dr) between 0.20 and 0.32 mm:
        !      V(Dr)=2831.*Dr**.80, V in m/s and Dr in m
        !      RR = PI*1000.*N0r0*2831.*Dr**(4+.80) = 7.115e13*Dr**4.80, 
        !        RR in kg/(m**2*s)
        !      Dr (m) = 1.3006e-3*RR**.2083 -> Dr (microns) = 1.3006e3*RR**.2083
        !
                INDEXR(I,J)=INT( 1.3006E3*RR(I,J)**.2083 + .5 )
                INDEXR(I,J)=MAX( MDR2, MIN(INDEXR(I,J), MDR3) )
              ELSE IF (RR(I,J) .LE. RR_DRmax) THEN
        !
        !--- Best fit to mass-weighted fall speeds (V) from rain lookup tables 
        !      for mean diameters (Dr) between 0.32 and 0.45 mm:
        !      V(Dr)=944.8*Dr**.6636, V in m/s and Dr in m
        !      RR = PI*1000.*N0r0*944.8*Dr**(4+.6636) = 2.3745e13*Dr**4.6636,
        !        RR in kg/(m**2*s)
        !      Dr (m) = 1.355e-3*RR**.2144 -> Dr (microns) = 1.355e3*RR**.2144
        !
                INDEXR(I,J)=INT( 1.355E3*RR(I,J)**.2144 + .5 )
                INDEXR(I,J)=MAX( MDR3, MIN(INDEXR(I,J), MDRmax) )
              ELSE 
        !
        !--- Assume fixed mean diameter of rain (0.45 mm) for high rain rates, 
        !      instead vary N0r with rain rate
        !
                INDEXR(I,J)=MDRmax
              ENDIF              ! End IF (RR .LE. RR_DRmin) etc. 
              VRAIN1(I,J)=GAMMAR(I,J)*VRAIN(INDEXR(I,J))
            ENDIF              ! End IF (ARAIN(I,J) .LE. 0.)
            TOT_RAIN(I,J)=THICK_COL(I,J,L)*QR_COL(I,J,L)+
     &           BLEND*ARAIN(I,J)
            QTRAIN(I,J)=TOT_RAIN(I,J)/
     &           (THICK_COL(I,J,L)+BLDTRH(I,J,L)*VRAIN1(I,J))
            PRLOSS(I,J)=-TOT_RAIN(I,J)/THICK_COL(I,J,L)
            RQR(I,J)=RHO(I,J,L)*QTRAIN(I,J)
   !
   !--- RQR - time-averaged rain content (kg/m**3)
   !
            IF (RQR(I,J) .LE. RQR_DRmin) THEN
              N0r(I,J)=MAX(N0rmin, CN0r_DMRmin*RQR(I,J))
              INDEXR(I,J)=MDRmin
            ELSE IF (RQR(I,J) .GE. RQR_DRmax) THEN
              N0r(I,J)=CN0r_DMRmax*RQR(I,J)
              INDEXR(I,J)=MDRmax
            ELSE
              N0r(I,J)=N0r0
              INDEXR(I,J)=MAX( XMRmin, MIN(CN0r0*RQR(I,J)**.25, XMRmax))
            ENDIF
   !
            IF (TC(I,J,L) .LT. T_ICE) THEN
              PIACR(I,J)=-PRLOSS(I,J)
            ELSE
              Dwvr(I,J)=WV(I,J,L)-PCOND(I,J)-QSW(I,J,L)
              DUM(I,J)=QW_COL(I,J,L)+PCOND(I,J)
              IF (Dwvr(I,J).LT.0. .AND. DUM(I,J).LE.CLIMIT) THEN
      !
      !--- Rain evaporation
      !
      !    * RFACTOR - [GAMMAR**.5]*[Schmidt**(1./3.)]*[(RHO/DYNVIS)**.5],
      !        where Schmidt (Schmidt Number) =DYNVIS/(RHO*DIFFUS)
      !
      !    * Units: RFACTOR - s**.5/m ;  ABW - m**2/s ;  VENTR - m**-2 ;  
      !             N0r - m**-4 ;  VENTR1 - m**2 ;  VENTR2 - m**3/s**.5 ;
      !             CREVP - unitless
      !
                RFACTOR=GAMMAR(I,J)**.5*(RHO(I,J,L)/
     &                (DIFFUS(I,J)*DIFFUS(I,J)*DYNVIS(i,j)))**C2
                ABW=1./(RHO(I,J,L)*XLV2(I,J)/THERM_COND(I,J)+
     &               1./DIFFUS(I,J))
      !
      !--- Note that VENTR1, VENTR2 lookup tables do not include the 
      !      1/Davg multiplier as in the ice tables
      !
                VENTR=N0r(I,J)*(VENTR1(INDEXR(I,J))+
     &               RFACTOR*VENTR2(INDEXR(I,J)))
                CREVP=ABW*VENTR*DTPH
                IF (CREVP .LT. Xratio) THEN
                  DUM(I,J)=Dwvr(I,J)*CREVP
                ELSE
                  DUM(I,J)=Dwvr(I,J)*(1.-EXP(-CREVP*DENOMW(I,J)))/
     &                  DENOMW(I,J)
                ENDIF
                PREVP(I,J)=MAX(DUM(I,J), PRLOSS(I,J))
              ELSE IF (QW_COL(I,J,L) .GT. CLIMIT) THEN
                FWR=CRACW*GAMMAR(I,J)*N0r(I,J)*ACCRR(INDEXR(I,J))
                PRACW(I,J)=MIN(1.,FWR)*QW_COL(I,J,L)
              ENDIF           ! End IF (DWVr.LT.0. .AND. DUM.LE.CLIMIT)
      !
              IF (TC(I,J,L).LT.0. .AND. TCC(I,J).LT.0.) THEN
         !
         !--- Biggs (1953) heteorogeneous freezing (e.g., Lin et al., 1983)
         !
                PIACR(I,J)=CBFR*N0r(I,J)*RRHO(I,J,L)*
     &                (EXP(ABFR*TC(I,J,L))-1.)*(FLOAT(INDEXR(I,J)))**7
                IF (QLICE(I,J) .GT. CLIMIT) THEN
            !
            !--- Freezing of rain by collisions w/ large ice
            !
                  DUM(I,J)=GAMMAR(I,J)*VRAIN(INDEXR(I,J))
                  DUM1(I,J)=DUM(I,J)-VSNOW(I,J)
            !
            !--- DUM2 - Difference in spectral fall speeds of rain and
            !      large ice, parameterized following eq. (48) on p. 112 of 
            !      Murakami (J. Meteor. Soc. Japan, 1990)
            !
                  DUM2(I,J)=(DUM1(I,J)*DUM1(I,J)+
     &                 .04*DUM(I,J)*VSNOW(I,J))**.5
                  DUM1(I,J)=5.E-12*INDEXR(I,J)*INDEXR(I,J)+
     &                 2.E-12*INDEXR(I,J)*INDEXS(I,J)
     &                 +.5E-12*INDEXS(I,J)*INDEXS(I,J)
                  FIR=MIN(1., CIACR*NLICE(I,J)*DUM1(I,J)*DUM2(I,J))
            !
            !--- Future?  Should COLLECTION BY SMALL ICE SHOULD BE INCLUDED???
            !
                  PIACR(I,J)=MIN(PIACR(I,J)+FIR*QTRAIN(I,J), 
     &                 QTRAIN(I,J))
                ENDIF        ! End IF (QLICE .GT. CLIMIT)
                DUM(I,J)=PREVP(I,J)-PIACR(I,J)
                If (DUM(I,J) .LT. PRLOSS(I,J)) THEN
                  DUM1(I,J)=PRLOSS(I,J)/DUM(I,J)
                  PREVP(I,J)=DUM1(I,J)*PREVP(I,J)
                  PIACR(I,J)=DUM1(I,J)*PIACR(I,J)
                ENDIF        ! End If (DUM .LT. PRLOSS)
              ENDIF          ! End IF (TC.LT.0. .AND. TCC.LT.0.)
            ENDIF            ! End IF (TC .LT. T_ICE)
          ENDIF              ! End IF (RAIN_logical) 
!
!----------------------------------------------------------------------
!---------------------- Main Budget Equations -------------------------
!----------------------------------------------------------------------
!
!
!-----------------------------------------------------------------------
!--- Update fields, determine characteristics for next lower layer ----
!-----------------------------------------------------------------------
!
!--- Carefully limit sinks of cloud water
!
          DUM1(I,J)=PIACW(I,J)+PRAUT(I,J)+PRACW(I,J)-MIN(0.,PCOND(I,J))
          IF (DUM1(I,J) .GT. QW_COL(I,J,L)) THEN
            DUM(I,J)=QW_COL(I,J,L)/DUM1(I,J)
            PIACW(I,J)=DUM(I,J)*PIACW(I,J)
            PIACWI(I,J)=DUM(I,J)*PIACWI(I,J)
            PRAUT(I,J)=DUM(I,J)*PRAUT(I,J)
            PRACW(I,J)=DUM(I,J)*PRACW(I,J)
            IF (PCOND(I,J) .LT. 0.) PCOND(I,J)=DUM(I,J)*PCOND(I,J)
          ENDIF
          PIACWR(I,J)=PIACW(I,J)-PIACWI(I,J)          ! TC >= 0C
!
!--- QWnew - updated cloud water mixing ratio
!
          DELW=PCOND(I,J)-PIACW(I,J)-PRAUT(I,J)-PRACW(I,J)
          Qwnew(I,J)=QW_COL(I,J,L)+DELW
          IF (Qwnew(I,J) .LE. CLIMIT) Qwnew(I,J)=0.
          IF (QW_COL(I,J,L).GT.0. .AND. Qwnew(I,J).NE.0.) THEN
            DUM(I,J)=Qwnew(I,J)/QW_COL(I,J,L)
            IF (DUM(I,J) .LT. TOLER) Qwnew(I,J)=0.
          ENDIF
!
!--- Update temperature and water vapor mixing ratios
!
          DELT= XLV1(I,J)*(PCOND(I,J)+PIEVP(I,J)+PICND(I,J)+PREVP(I,J))
     &         +XLS1*PIDEP(I,J)+XLF1(I,J)*
     &         (PIACWI(I,J)+PIACR(I,J)-PIMLT(I,J))
          Tnew(I,J)=T_COL(I,J,L)+DELT
!
          DELV=-PCOND(I,J)-PIDEP(I,J)-PIEVP(I,J)-PICND(I,J)-PREVP(I,J)
          Wvnew(I,J)=WV(I,J,L)+DELV
!
!--- Update ice mixing ratios
!
!---
!  * TOT_ICEnew - total mass (small & large) ice after microphysics,
!                 which is the sum of the total mass of large ice in the 
!                 current layer and the flux of ice out of the grid box below
!  * RimeF      - Rime Factor, which is the mass ratio of total (unrimed & 
!                 rimed) ice mass to the unrimed ice mass (>=1)
!  * QInew      - updated mixing ratio of total (large & small) ice in layer
!      -> TOT_ICEnew=QInew*THICK+BLDTRH*QLICEnew*VSNOW
!        -> But QLICEnew=QInew*FLIMASS, so
!      -> TOT_ICEnew=QInew*(THICK+BLDTRH*FLIMASS*VSNOW)
!  * ASNOWnew   - updated accumulation of snow at bottom of grid cell
!---
!
          DELI=0.
          Rimef(I,J)=1.
          IF (ice_logical(i,j)) THEN
            DELI=PIDEP(I,J)+PIEVP(I,J)+PIACWI(I,J)+PIACR(I,J)-PIMLT(I,J)
            Tot_Icenew(I,J)=TOT_ICE(I,J)+THICK_COL(I,J,L)*DELI
            IF (TOT_ICE(I,J).GT.0. .AND. Tot_Icenew(I,J).NE.0.) THEN
              DUM(I,J)=Tot_Icenew(I,J)/TOT_ICE(I,J)
              IF (DUM(I,J) .LT. TOLER) Tot_Icenew(I,J)=0.
            ENDIF
            IF (Tot_Icenew(I,J) .LE. CLIMIT) THEN
              Tot_Icenew(I,J)=0.
              Rimef(I,J)=1.
              Qinew(I,J)=0.
              Asnownew(I,J)=0.
            ELSE
      !
      !--- Update rime factor if appropriate
      !
              DUM(I,J)=PIACWI(I,J)+PIACR(I,J)
              IF (DUM(I,J).LE.CLIMIT .AND. PIDEP(I,J).LE.CLIMIT) THEN
                Rimef(I,J)=Rimef1(I,J)
              ELSE
         !
         !--- Rime Factor, RimeF = (Total ice mass)/(Total unrimed ice mass)
         !      DUM1 - Total ice mass, rimed & unrimed
         !      DUM2 - Estimated mass of *unrimed* ice
         !
                DUM1(I,J)=TOT_ICE(I,J)+THICK_COL(I,J,L)*
     &                (PIDEP(I,J)+DUM(I,J))
                DUM2(I,J)=TOT_ICE(I,J)/Rimef1(I,J)+
     &               THICK_COL(I,J,L)*PIDEP(I,J)
                IF (DUM2(I,J) .LE. 0.) THEN
                  Rimef(I,J)=RFmax
                ELSE
                  Rimef(I,J)=MIN(RFmax, MAX(1., DUM1(I,J)/DUM2(I,J)) )
                ENDIF
              ENDIF       ! End IF (DUM.LE.CLIMIT .AND. PIDEP(I,J).LE.CLIMIT)
              Qinew(I,J)=Tot_Icenew(I,J)/(THICK_COL(I,J,L)+
     &             BLDTRH(I,J,L)*FLIMASS(I,J)*VSNOW(I,J))
              IF (Qinew(I,J) .LE. CLIMIT) Qinew(I,J)=0.
              IF (QI_COL(I,J,L).GT.0. .AND. Qinew(I,J).NE.0.) THEN
                DUM(I,J)=Qinew(I,J)/QI_COL(I,J,L)
                IF (DUM(I,J) .LT. TOLER) Qinew(I,J)=0.
              ENDIF
              Asnownew(I,J)=BLDTRH(I,J,L)*FLIMASS(I,J)*
     &             VSNOW(I,J)*Qinew(I,J)
              IF (ASNOW(I,J).GT.0. .AND. Asnownew(I,J).NE.0.) THEN
                DUM(I,J)=Asnownew(I,J)/ASNOW(I,J)
                IF (DUM(I,J) .LT. TOLER) Asnownew(I,J)=0.
              ENDIF
            ENDIF         ! End IF (TOT_ICEnew .LE. CLIMIT)
          ENDIF           ! End IF (ICE_logical)
!
!--- Update rain mixing ratios
!
!---
! * TOT_RAINnew - total mass of rain after microphysics
!                 current layer and the input flux of ice from above
! * VRAIN2      - time-averaged fall speed of rain in grid and below 
!                 (with air resistance correction)
! * QRnew       - updated rain mixing ratio in layer
!      -> TOT_RAINnew=QRnew*(THICK+BLDTRH*VRAIN2)
!  * ARAINnew  - updated accumulation of rain at bottom of grid cell
!---
!
          DELR=PRAUT(I,J)+PRACW(I,J)+PIACWR(I,J)-PIACR(I,J)+PIMLT(I,J)+
     &         PREVP(I,J)+PICND(I,J)
          Tot_Rainnew(I,J)=TOT_RAIN(I,J)+THICK_COL(I,J,L)*DELR
          IF (TOT_RAIN(I,J).GT.0. .AND. Tot_Rainnew(I,J).NE.0.) THEN
            DUM(I,J)=Tot_Rainnew(I,J)/TOT_RAIN(I,J)
            IF (DUM(I,J) .LT. TOLER) Tot_Rainnew(I,J)=0.
          ENDIF
          IF (Tot_Rainnew(I,J) .LE. CLIMIT) THEN
            Tot_Rainnew(I,J)=0.
            VRAIN2(I,J)=0.
            Qrnew(I,J)=0.
            Arainnew(I,J)=0.
          ELSE
   !
   !--- 1st guess time-averaged rain rate at bottom of grid box
   !
            RR(I,J)=Tot_Rainnew(I,J)/(DTPH*GAMMAR(I,J))
   !
   !--- Use same algorithm as above for calculating mean drop diameter
   !      (IDR, in microns), which is used to estimate the time-averaged
   !      fall speed of rain drops at the bottom of the grid layer.  This
   !      isnt perfect, but the alternative is solving a transcendental 
   !      equation that is numerically inefficient and nasty to program
   !      (coded in earlier versions of GSMCOLUMN prior to 8-22-01).
   !
            IF (RR(I,J) .LE. RR_DRmin) THEN
              IDR=MDRmin
            ELSE IF (RR(I,J) .LE. RR_DR1) THEN
              IDR=INT( 1.123E3*RR(I,J)**.1947 + .5 )
              IDR=MAX( MDRmin, MIN(IDR, MDR1) )
            ELSE IF (RR(I,J) .LE. RR_DR2) THEN
              IDR=INT( 1.225E3*RR(I,J)**.2017 + .5 )
              IDR=MAX( MDR1, MIN(IDR, MDR2) )
            ELSE IF (RR(I,J) .LE. RR_DR3) THEN
              IDR=INT( 1.3006E3*RR(I,J)**.2083 + .5 )
              IDR=MAX( MDR2, MIN(IDR, MDR3) )
            ELSE IF (RR(I,J) .LE. RR_DRmax) THEN
              IDR=INT( 1.355E3*RR(I,J)**.2144 + .5 )
              IDR=MAX( MDR3, MIN(IDR, MDRmax) )
            ELSE 
              IDR=MDRmax
            ENDIF              ! End IF (RR .LE. RR_DRmin)
            VRAIN2(I,J)=GAMMAR(I,J)*VRAIN(IDR)
            Qrnew(I,J)=Tot_Rainnew(I,J)/
     &           (THICK_COL(I,J,L)+BLDTRH(I,J,L)*VRAIN2(I,J))
            IF (Qrnew(I,J) .LE. CLIMIT) Qrnew(I,J)=0.
            IF (QR_COL(I,J,L).GT.0. .AND. Qrnew(I,J).NE.0.) THEN
              DUM(I,J)=Qrnew(I,J)/QR_COL(I,J,L)
              IF (DUM(I,J) .LT. TOLER) Qrnew(I,J)=0.
            ENDIF
            Arainnew(I,J)=BLDTRH(I,J,L)*VRAIN2(I,J)*Qrnew(I,J)
            IF (ARAIN(I,J).GT.0. .AND. Arainnew(I,J).NE.0.) THEN
              DUM(I,J)=Arainnew(I,J)/ARAIN(I,J)
              IF (DUM(I,J) .LT. TOLER) Arainnew(I,J)=0.
            ENDIF
          ENDIF
!
          Wcnew(I,J)=Qwnew(I,J)+Qrnew(I,J)+Qinew(I,J)
!
!----------------------------------------------------------------------
!-------------- Begin debugging & verification ------------------------
!----------------------------------------------------------------------
!
!--- QT, QTnew - total water (vapor & condensate) before & after microphysics, resp.
!
          QT=THICK_COL(I,J,L)*(WV(I,J,L)+WC_COL(I,J,L))+
     &         ARAIN(I,J)+ASNOW(I,J)
          QTnew=THICK_COL(I,J,L)*(Wvnew(I,J)+Wcnew(I,J))+
     &         Arainnew(I,J)+Asnownew(I,J)
          BUDGET=QT-QTnew
!
!--- Additional check on budget preservation, accounting for truncation effects
!
          DBG_logical=.FALSE.
!          DUM=ABS(BUDGET)
!          IF (DUM .GT. TOLER) THEN
!            DUM=DUM/MIN(QT, QTnew)
!            IF (DUM .GT. TOLER) DBG_logical=.TRUE.
!          ENDIF
!!
!          DUM=(RHgrd+.001)*QSInew
!          IF ( (Qwnew(I,J).GT.CLIMIT .OR. Qrnew(I,J).GT.CLIMIT .OR. 
!     &         Wvnew(I,J).GT.DUM)
!     &        .AND. TC(I,J,L).LT.T_ICE )  DBG_logical=.TRUE.
!
!          IF (TC(I,J,L).GT.5. .AND. QInew.GT.CLIMIT) DBG_logical=.TRUE.
!
          IF ((Wvnew(I,J).LT.CLIMIT .OR. DBG_logical) 
     &         .AND. PRINT_diag) THEN
   !
   !
            ESW=1000.*FPVS0(Tnew(I,J))
            QSWnew=EPS*ESW/(P_COL(I,J,L)-ESW)
            IF (TC(I,J,L).LT.0. .OR. Tnew(I,J) .LT. 0.) THEN
              ESI=1000.*FPVS(Tnew(I,J))
              QSInew=EPS*ESI/(P_COL(I,J,L)-ESI)
            ELSE
              QSI(I,J,L)=QSW(I,J,L)
              QSInew=QSWnew
            ENDIF
            WSnew=QSInew
   !
   !
            DUM(I,J)=PIMLT(I,J)+PICND(I,J)-PREVP(I,J)-PIEVP(I,J)
   !
   !
            DUM(I,J)=PIMLT(I,J)+PICND(I,J)-PIEVP(I,J)
   !
          ENDIF
!
!----------------------------------------------------------------------
!-------------- Water budget statistics & maximum values --------------
!----------------------------------------------------------------------
!
          IF (PRINT_diag) THEN
!**(JP)**
            ITdx=MAX( ITLO, MIN( INT(Tnew(I,J)-T0C), ITHI ) )
            IF (Qinew(I,J) .GT. CLIMIT) NSTATS(ITdx,1)=NSTATS(ITdx,1)+1
            IF (Qinew(I,J).GT.CLIMIT .AND.  
     &           Qrnew(I,J)+Qwnew(I,J).GT.CLIMIT)
     &        NSTATS(ITdx,2)=NSTATS(ITdx,2)+1
            IF (Qwnew(I,J) .GT. CLIMIT) NSTATS(ITdx,3)=NSTATS(ITdx,3)+1
            IF (Qrnew(I,J) .GT. CLIMIT) NSTATS(ITdx,4)=NSTATS(ITdx,4)+1
  !
            QMAX(ITdx,1)=MAX(QMAX(ITdx,1), Qinew(I,J))
            QMAX(ITdx,2)=MAX(QMAX(ITdx,2), Qwnew(I,J))
            QMAX(ITdx,3)=MAX(QMAX(ITdx,3), Qrnew(I,J))
            QMAX(ITdx,4)=MAX(QMAX(ITdx,4), Asnownew(I,J))
            QMAX(ITdx,5)=MAX(QMAX(ITdx,5), Arainnew(I,J))
            QTOT(ITdx,1)=QTOT(ITdx,1)+Qinew(I,J)*THICK_COL(I,J,L)
            QTOT(ITdx,2)=QTOT(ITdx,2)+Qwnew(I,J)*THICK_COL(I,J,L)
            QTOT(ITdx,3)=QTOT(ITdx,3)+Qrnew(I,J)*THICK_COL(I,J,L)
  !
            QTOT(ITdx,4)=QTOT(ITdx,4)+PCOND(I,J)*THICK_COL(I,J,L)
            QTOT(ITdx,5)=QTOT(ITdx,5)+PICND(I,J)*THICK_COL(I,J,L)
            QTOT(ITdx,6)=QTOT(ITdx,6)+PIEVP(I,J)*THICK_COL(I,J,L)
            QTOT(ITdx,7)=QTOT(ITdx,7)+PIDEP(I,J)*THICK_COL(I,J,L)
            QTOT(ITdx,8)=QTOT(ITdx,8)+PREVP(I,J)*THICK_COL(I,J,L)
            QTOT(ITdx,9)=QTOT(ITdx,9)+PRAUT(I,J)*THICK_COL(I,J,L)
            QTOT(ITdx,10)=QTOT(ITdx,10)+PRACW(I,J)*THICK_COL(I,J,L)
            QTOT(ITdx,11)=QTOT(ITdx,11)+PIMLT(I,J)*THICK_COL(I,J,L)
            QTOT(ITdx,12)=QTOT(ITdx,12)+PIACW(I,J)*THICK_COL(I,J,L)
            QTOT(ITdx,13)=QTOT(ITdx,13)+PIACWI(I,J)*THICK_COL(I,J,L)
            QTOT(ITdx,14)=QTOT(ITdx,14)+PIACWR(I,J)*THICK_COL(I,J,L)
            QTOT(ITdx,15)=QTOT(ITdx,15)+PIACR(I,J)*THICK_COL(I,J,L)
  !
            QTOT(ITdx,16)=QTOT(ITdx,16)+(Wvnew(I,J)-WV(I,J,L))*
     &           THICK_COL(I,J,L)
            QTOT(ITdx,17)=QTOT(ITdx,17)+
     &           (Qwnew(I,J)-QW_COL(I,J,L))*THICK_COL(I,J,L)
            QTOT(ITdx,18)=QTOT(ITdx,18)+
     &           (Qinew(I,J)-QI_COL(I,J,L))*THICK_COL(I,J,L)
            QTOT(ITdx,19)=QTOT(ITdx,19)+
     &           (Qrnew(I,J)-QR_COL(I,J,L))*THICK_COL(I,J,L)
            QTOT(ITdx,20)=QTOT(ITdx,20)+(Arainnew(I,J)-ARAIN(I,J))
            QTOT(ITdx,21)=QTOT(ITdx,21)+(Asnownew(I,J)-ASNOW(I,J))
            IF (Qinew(I,J) .GT. 0.) 
     &        QTOT(ITdx,22)=QTOT(ITdx,22)+
     &           Qinew(I,J)*THICK_COL(I,J,L)/Rimef(I,J)
  !
          ENDIF
!
!----------------------------------------------------------------------
!------------------------- Update arrays ------------------------------
!----------------------------------------------------------------------
!
          t_col(i,j,l)=Tnew(I,J)                           ! Updated temperature
!
          qv_col(i,j,l)=max(EPSQ, Wvnew(I,J)/(1.+Wvnew(I,J)))   ! Updated specific humidity
          wc_col(i,j,l)=max(EPSQ, Wcnew(I,J))              ! Updated total condensate mixing ratio
          qi_col(i,j,l)=max(EPSQ, Qinew(I,J))              ! Updated ice mixing ratio
          qr_col(i,j,l)=max(EPSQ, Qrnew(I,J))              ! Updated rain mixing ratio
          qw_col(i,j,l)=max(EPSQ, Qwnew(I,J))              ! Updated cloud water mixing ratio
          rimef_col(i,j,l)=Rimef(I,J)                      ! Updated rime factor
          ASNOW(I,J)=Asnownew(I,J)                          ! Updated accumulated snow
          ARAIN(I,J)=Arainnew(I,J)                          ! Updated accumulated rain
!
!#######################################################################
!
       END DO
       END DO
       END DO
!
!#######################################################################
!
!-----------------------------------------------------------------------
!--------------------------- Return to GSMDRIVE -----------------------
!-----------------------------------------------------------------------
!
      RETURN
      END
!
!#######################################################################
!--------- Produces accurate calculation of cloud condensation ---------
!#######################################################################
!
      REAL FUNCTION CONDENSE (PP, QW, RHgrd, TK, WV)
!
!---------------------------------------------------------------------------------
!------   The Asai (1965) algorithm takes into consideration the release of ------
!------   latent heat in increasing the temperature & in increasing the     ------
!------   saturation mixing ratio (following the Clausius-Clapeyron eqn.).  ------
!---------------------------------------------------------------------------------
!
      INTEGER, PARAMETER :: HIGH_PRES=Selected_Real_Kind(15)
      REAL (KIND=HIGH_PRES), PARAMETER :: CLIMIT=1.E-20,
     & RHLIMIT=.001, RHLIMIT1=-RHLIMIT
      REAL, PARAMETER :: CP=1004.6, RD=287.04, RV=461.5, EPS=RD/RV, 
     & RCP=1./CP, RCPRV=RCP/RV
      REAL (KIND=HIGH_PRES) :: COND, SSAT, WCdum
!
!-----------------------------------------------------------------------
!
!--- LV (T) is from Bolton (JAS, 1980)
!
      XLV=3.148E6-2370.*TK
      XLV1=XLV*RCP
      XLV2=XLV*XLV*RCPRV
      Tdum=TK
      WVdum=WV
      WCdum=QW
      ESW=1000.*FPVS0(Tdum)                     ! Saturation vapor press w/r/t water
      WS=RHgrd*EPS*ESW/(PP-ESW)                 ! Saturation mixing ratio w/r/t water
      DWV=WVdum-WS                              ! Deficit grid-scale water vapor mixing ratio
      SSAT=DWV/WS                               ! Supersaturation ratio
      CONDENSE=0.
      DO WHILE ((SSAT.LT.RHLIMIT1 .AND. WCdum.GT.CLIMIT)
     &           .OR. SSAT.GT.RHLIMIT)
        COND=DWV/(1.+XLV2*WS/(Tdum*Tdum))       ! Asai (1965, J. Japan)
        COND=MAX(COND, -WCdum)                  ! Limit cloud water evaporation
        Tdum=Tdum+XLV1*COND                     ! Updated temperature
        WVdum=WVdum-COND                        ! Updated water vapor mixing ratio
        WCdum=WCdum+COND                        ! Updated cloud water mixing ratio
        CONDENSE=CONDENSE+COND                  ! Total cloud water condensation
        ESW=1000.*FPVS0(Tdum)                   ! Updated saturation vapor press w/r/t water
        WS=RHgrd*EPS*ESW/(PP-ESW)               ! Updated saturation mixing ratio w/r/t water
        DWV=WVdum-WS                            ! Deficit grid-scale water vapor mixing ratio
        SSAT=DWV/WS                             ! Grid-scale supersaturation ratio
      ENDDO
      RETURN
      END
!
!#######################################################################
!---------------- Calculate ice deposition at T<T_ICE ------------------
!#######################################################################
!
      REAL FUNCTION DEPOSIT (PP, RHgrd, Tdum, WVdum)
!
!--- Also uses the Asai (1965) algorithm, but uses a different target
!      vapor pressure for the adjustment
!
      INTEGER, PARAMETER :: HIGH_PRES=Selected_Real_Kind(15)
      REAL (KIND=HIGH_PRES), PARAMETER :: RHLIMIT=.001, 
     & RHLIMIT1=-RHLIMIT
      REAL, PARAMETER :: CP=1004.6, RD=287.04, RV=461.5, XLS=2.834E6, 
     & EPS=RD/RV, RCP=1./CP, RCPRV=RCP/RV, XLS1=XLS*RCP, 
     & XLS2=XLS*XLS*RCPRV
      REAL (KIND=HIGH_PRES) :: DEP, SSAT
!
!-----------------------------------------------------------------------
!
      ESI=1000.*FPVS(Tdum)                      ! Saturation vapor press w/r/t ice
      WS=RHgrd*EPS*ESI/(PP-ESI)                 ! Saturation mixing ratio
      DWV=WVdum-WS                              ! Deficit grid-scale water vapor mixing ratio
      SSAT=DWV/WS                               ! Supersaturation ratio
      DEPOSIT=0.
      DO WHILE (SSAT.GT.RHLIMIT .OR. SSAT.LT.RHLIMIT1)
   !
   !--- Note that XLVS2=LS*LV/(CP*RV)=LV*WS/(RV*T*T)*(LS/CP*DEP1), 
   !     where WS is the saturation mixing ratio following Clausius-
   !     Clapeyron (see Asai,1965; Young,1993,p.405) 
   !
        DEP=DWV/(1.+XLS2*WS/(Tdum*Tdum))        ! Asai (1965, J. Japan)
        Tdum=Tdum+XLS1*DEP                      ! Updated temperature
        WVdum=WVdum-DEP                         ! Updated ice mixing ratio
        DEPOSIT=DEPOSIT+DEP                     ! Total ice deposition
        ESI=1000.*FPVS(Tdum)                    ! Updated saturation vapor press w/r/t ice
        WS=RHgrd*EPS*ESI/(PP-ESI)               ! Updated saturation mixing ratio w/r/t ice
        DWV=WVdum-WS                            ! Deficit grid-scale water vapor mixing ratio
        SSAT=DWV/WS                             ! Grid-scale supersaturation ratio
      ENDDO
      RETURN
      END
